@@ -1,4 +1,4 @@
-from typing import Dict, Iterator, List, Optional
+from typing import Dict, Iterator, List, Optional, Tuple
 
 import requests
 from pydantic import parse_obj_as
@@ -9,6 +9,9 @@ from contaxy.schema import File, FileInput, FileStream, ResourceAction
 
 
 class FileClient(FileOperations):
+
+    _FILE_METADATA_PREFIX = "x-amz-meta-"
+
     def __init__(self, client: requests.Session):
         self._client = client
 
@@ -82,6 +85,7 @@ class FileClient(FileOperations):
         project_id: str,
         file_key: str,
         file_stream: FileStream,
+        metadata: Optional[Dict[str, str]] = None,
         content_type: str = "application/octet-stream",
         request_kwargs: Dict = {},
     ) -> File:
@@ -89,10 +93,20 @@ class FileClient(FileOperations):
 
         # TODO Check if it actually gets streamed or not
         # ! In the event you are posting a very large file as a multipart/form-data request, you may want to stream the request. By default, requests does not support this, but there is a separate package which does - requests-toolbelt. You should read the toolbelt’s documentation for more details about how to use it.
+        processed_metadata = {}
+
+        if metadata:
+            # Add metadata as headers, to be valid, metadata needs to have the x-amz-meta prefix (S3 conform)
+            for key in metadata:
+                value = metadata[key]
+                if not key.startswith(self._FILE_METADATA_PREFIX):
+                    key = self._FILE_METADATA_PREFIX + key
+                processed_metadata[key] = value
 
         response = self._client.post(
             f"/projects/{project_id}/files/{file_key}",
             files={"file": (f"{file_key}", file_stream, content_type)},
+            headers=processed_metadata,
             **request_kwargs,
         )
         handle_errors(response)
@@ -104,7 +118,7 @@ class FileClient(FileOperations):
         file_key: str,
         version: Optional[str] = None,
         request_kwargs: Dict = {},
-    ) -> Iterator[bytes]:
+    ) -> Tuple[Iterator[bytes], int]:
         query_params: Dict = {}
         if version:
             query_params.update({"version": version})
@@ -116,7 +130,9 @@ class FileClient(FileOperations):
             **request_kwargs,
         )
         handle_errors(response)
-        return response.iter_content(chunk_size=10 * 1024 * 1024)
+        return response.iter_content(chunk_size=10 * 1024 * 1024), int(
+            response.headers.get("Content-Length", 0)
+        )
 
     def delete_file(
         self,
